from collections.abc import Iterable

from spb import MB
from spb.backends.matplotlib.renderers import Line2DRenderer
from spb.graphics import graphics
from spb.series import List2DSeries


def _check_len_annotations(x, labels):
    if len(labels) != len(x):
        msg = "Number of annotations should be equal to the number of points."
        raise ValueError(msg)


class Point2DSeries(List2DSeries):
    """List of points (scatter plot)."""

    def __init__(self, list_x, list_y, label="", annotations=None, **kwargs):
        kwargs.setdefault("is_point", True)

        if annotations is not None:
            _check_len_annotations(list_x, annotations)
        self.annotations = annotations

        super().__init__(list_x, list_y, label=label, **kwargs)


class Point2DRenderer(Line2DRenderer):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def draw(self):
        s = self.series
        p = self.plot
        data = s.get_data()
        if s.annotations is not None and isinstance(p, MB):
            for x, y, annotation in zip(*data, s.annotations):
                p._ax.annotate(annotation, (x, y))
        super().draw()


MB.renderers_map.update({Point2DSeries: Point2DRenderer})  # type: ignore


def plot_points(x: Iterable, y: Iterable, label="", annotations=None, **kwargs):
    """Afbild punkter i et koordinatsystem

        `plot_points(x,y)`

    - Afbild punkter med angivelse af label og rendering_kw

        `plot_points(x,y,label,rendering_kw)`

    Parametre
    ---------
    x : Iterable
        Punkternes x-værdier

    y : Iterable
        Punkternes y-værdier

    label : String, optional
        Label for alle punkter

    annotations : Iterable, optional
        Annotering for alle punkter. Vises som tekst ved hvert punkt.

    rendering_kw : se `list_2d`
    kwargs : se `list_2d`

    Returnerer
    ---------
    plt : Plot
        Plotobjektet.

    Se også
    ---------
    - [SPB: list_2d](https://sympy-plot-backends.readthedocs.io/en/latest/modules/graphics/functions_2d.html#spb.graphics.functions_2d.list_2d)
    """
    if isinstance(label, (tuple, list)):
        label = label[0]
    return graphics(Point2DSeries(x, y, label, annotations, **kwargs), **kwargs)


if __name__ == "__main__":
    # a = plot_points([1, 2], [0, 10], label="as", show=False)
    # b = plot_points([2, 3], [0, 10], label="as2", show=False)
    # if isinstance(a, MB) and isinstance(b, MB):
    #     (a + b).show()

    # p = plot_points([1, 2, 3, 3], [1, 3, 5, 7])

    # plot_points([-4, 0], [-2, 5], "", rendering_kw={"marker": "*"})

    a = plot_points(
        [-40, 0, 0],
        [-20, 50, 100],
        "hej",
        annotations=["A", "B", "C"],
        rendering_kw={"marker": "*"},
        aspect="equal",
        show=False,
    )
    b = plot_points(
        [-4, 0], [-2, 5], "hej", rendering_kw={"marker": "*"}, aspect="equal", annotations=["A", "B"], show=False
    )
    if isinstance(a, MB) and isinstance(b, MB):
        (a + b).show()
